<?php

declare(strict_types=1);

namespace NOUT\Soap\PackageBase;

use InvalidArgumentException;
use JsonSerializable;
use ReflectionClass;

abstract class AbstractStructBase implements StructInterface, JsonSerializable
{
    /**
     * Returns the properties of this object
     * @return mixed[]
     */
    public function jsonSerialize(): array
    {
        return \get_object_vars($this);
    }

    /**
     * Generic method called when an object has been exported with var_export() functions
     * It allows to return an object instantiated with the values
     * @param array $array the exported values
     * @return self
     */
    public static function __set_state(array $array): StructInterface
    {
        $reflection = new ReflectionClass(get_called_class());
        $object = $reflection->newInstance();
        foreach ($array as $name => $value) {
            $object->setPropertyValue($name, $value);
        }

        return $object;
    }

    /**
     * Generic method setting value
     * @throws InvalidArgumentException
     * @param string $name property name to set
     * @param mixed $value property value to use
     * @return self
     * @internal
     */
    public function setPropertyValue(string $name, $value): self
    {
        $setMethod = 'set' . ucfirst($name);
        if (method_exists($this, $setMethod)) {
            $this->$setMethod($value);
        } else {
            throw new InvalidArgumentException(sprintf('Setter does not exist for "%s" property', $name));
        }

        return $this;
    }

    /**
     * Generic method getting value
     * @throws InvalidArgumentException
     * @param string $name property name to get
     * @return mixed
     * @internal
     */
    public function getPropertyValue(string $name)
    {
        $getMethod = 'get' . ucfirst($name);
        if (method_exists($this, $getMethod)) {
            return $this->$getMethod();
        }

        throw new InvalidArgumentException(sprintf('Getter does not exist for "%s" property', $name));
    }

    /**
     * Default string representation of current object. Don't want to expose any sensible data
     * @return string
     */
    public function __toString(): string
    {
        return get_called_class();
    }

    public function isOfType(array $aStruct) : bool
    {
        foreach(get_object_vars($this) as $name => $value)
        {
            if (is_null($value)){
                continue;
            }

            $type = $aStruct[$name]['type'];
            if ($type[0]=='\\'){
                //c'est une classe, on vérifie si c'est la valeur est de la bonne classe
                if (!($value instanceof $type)){
                    return false; //pas le bon type
                }

                $subdef = $aStruct[$name]['subdef'];
                if (!is_null($subdef) && ($value instanceof StructInterface) && !$value->isOfType($subdef))
                {
                    return false;
                }
            }
            else {
                $method = "is_$type";
                if (!call_user_func($method, $value)){
                    return false;
                }
            }
        }


        return true;
    }
}
