<?php

/**
 * Classe contenant les constantes (enum) pour les options du plugin SimaxFormSubmitter.
 */
class sfs_clEnumOption
{
    // Connexion au webService
    const sTypeAdresse = 'sfs_typeaddress';
    const sHost        = 'sfs_host';
    const sPort        = 'sfs_port';
    const sUrl         = 'sfs_serviceEndpoint';
    const sUserApiuuId = 'sfs_apiuuid';

    // Option Traces
    const bTraces      = 'sfs_traces';

    //Token / phrase secrète
    const sToken = 'sfs_token';
    const sTokenTime = 'sfs_tokenTime';
    const sSecretPhrase = 'sfs_secretPhrase';

    // Email de récupération d'erreur
    const sEmailRecupError = 'sfs_mailErrorRecup';

    // Messages d'erreur/réussite
    const sErrorMessage = 'sfs_errorMessage';
    const sSuccessMessage = 'sfs_successMessage';
}

/**
 * Classe de gestion des options de configuration du plugin SimaxFormSubmitter.
 * Cette classe doit être exécutée dans un contexte WordPress.
 */
class sfs_clSettingsGest
{
    /**
     * Constructeur de la classe.
     */
    public function __construct()
    {
        // Constructeur vide pour le moment.
    }

    /********************************************************************************************************/
    /******* Fonctions concernant les options du plugin *****************************************************/

    /**
     * Génère un tableau contenant tous les noms d'options WordPress du plugin (SimaxFormSubmitter).
     *
     * @return array Tableau de toutes les constantes définies dans sfs_clEnumOption.
     */
    public function list_options_name()
    {
        $oReflection = new ReflectionClass('sfs_clEnumOption');
        return $oReflection->getConstants();
    }

    /**
     * Vérifie si une option existe.
     *
     * @param string $sOptName Nom de l'option.
     * @return bool True si l'option existe, False sinon.
     */
    public function option_exist($sOptName)
    {
        if (!is_string($sOptName)) {
            return false;
        }

        $aOpts = array_flip($this->list_options_name());
        return isset($aOpts[$sOptName]);
    }

    /********************************************************************************************************/
    /******* Fonctions concernant les valeurs d'option du plugin ********************************************/

    /**
     * Initialise toutes les options du plugin (SimaxFormSubmitter).
     *
     * @param array $aDefaultValues (Optionnel) Valeurs par défaut des options à définir ('nom de l'option' => 'valeur par défaut').
     */
    public function init_plugin_opts($aDefaultValues = [])
    {
        if (!is_array($aDefaultValues)) {
            $aDefaultValues = [];
        }

        $aOptNames = $this->list_options_name();

        foreach ($aOptNames as $sOptToInit) {
            $sDefaultValue = isset($aDefaultValues[$sOptToInit]) ? $aDefaultValues[$sOptToInit] : '';
            add_option($sOptToInit, $sDefaultValue);
        }
    }

    /**
     * Supprime toutes les options du plugin.
     */
    public function remove_plugin_opt()
    {
        $aOptNames = $this->list_options_name();

        foreach ($aOptNames as $sOptToDel) {
            delete_option($sOptToDel);
        }
    }

    /**
     * Génère un tableau de description de toutes les options du plugin WordPress (SimaxFormSubmitter).
     *
     * @return array Tableau des valeurs des options ('Nom de l'option WP' => 'valeur de l'option WP').
     */
    public function list_options_value(array $aValDef=[])
    {
        $aListOpts = $this->list_options_name();
        $aPluginOptVals = [];

        foreach ($aListOpts as $sOptName) {
            $aPluginOptVals[$sOptName] = $this->get_option_value($sOptName, $aValDef[$sOptName] ?? false);
        }

        return $aPluginOptVals;
    }

    /**
     * Met à jour plusieurs options du plugin (SimaxFormSubmitter).
     *
     * @param array $aOptions Liste des options à mettre à jour ('nom de l'option wordpress' => 'nouvelle valeur').
     * @return bool True si une ou plusieurs options ont été mises à jour, False sinon.
     */
    public function update_multiple_option($aOptions = [])
    {
        if (!is_array($aOptions)) {
            return false;
        }

        if (!array_key_exists(sfs_clEnumOption::bTraces, $aOptions)){
            $aOptions[sfs_clEnumOption::bTraces]="0";
        }

        $aListOptNames = array_flip($this->list_options_name());
        $bOneOrMoreUpdate = false;

        foreach ($aOptions as $sKey => $sVal) {
            if (isset($aListOptNames[$sKey]) && $aListOptNames[$sKey] != '') {
                if (update_option($sKey, trim($sVal))) {
                    $bOneOrMoreUpdate = true;
                }
            }
        }

        return $bOneOrMoreUpdate;
    }

    /********************************************************************************************************/
    /******* Fonctions concernant directement les appels aux fonctions WP ***********************************/

    /**
     * Récupère la valeur d'une option du plugin SimaxFormSubmitter.
     *
     * @param string $sOptionName Nom de l'option dont on veut récupérer la valeur.
     * @param bool   $valdefault valeur renvoyée si pas de valeur par défaut
     *
     * @return mixed False en cas d'erreur, valeur de l'option sinon.
     */
    public function get_option_value($sOptionName, $valdefault = false)
    {
        if (!$this->option_exist($sOptionName)) {
            return $valdefault;
        }

        return get_option($sOptionName, $valdefault);
    }
}
?>
